;; ns-icon
;; Description:  Automagically updates the icon associated with each frame
;;		 as the file currently being edited in that frame changes.
;;		 Really only useful under Emacs 19 for NeXTSTEP.
;; Author:    Joe Reiss			<jreiss@magnus.acs.ohio-state.edu>
;; File:      ns-icon.el
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or 
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

;; LCD Archive Entry:
;; ns-icon|Joe Reiss|jreiss@magnus.acs.ohio-state.edu|
;; Update frame icons based on name of file currently being edited.|
;; 11-June-1994|1.00||

;; INSTALLATION: 
;; byte-compile it if you must, then put this file in one of your lisp
;; load directories and add the following line to your .emacs file
;;
;; (require 'ns-icon)

(require 'advice)
(provide 'ns-icon)

(defvar ns-icon-type-alist 
  '(("\\.bib$" . "bib") ("\\.c$" . "c")
    ("\\.el$"  . "el")  ("\\.h$" . "h")
    ("\\.l$"   . "l")   ("\\.m$" . "m")
    ("\\.tex$" . "tex") ("\\.y$" . "y"))
"Alist mapping file names to icons.

Each element is a cons cell containing a pattern to match against the
current filename and the type of the icon to be used if the filename
matches that pattern.")

(defun ns-set-frame-icon ()
"Update the current frame's icon based on the name of the file being edited.
The icon type to be used is determined from the variable `ns-icon-type-alist'."
  (interactive)
  (let* ((frame (selected-frame))
	 (filename (buffer-file-name 
		    (window-buffer (frame-selected-window frame))))
	 (aptr (append ns-icon-type-alist '(("*" . nil))))
	 (pattern) (icon-form))
    (if filename
	(progn
	  (while aptr
	    (setq pattern (car (car aptr)))
	    (setq icon-form (cdr (car aptr)))
	    (if (string-match pattern filename)
		(setq aptr nil)
	      (setq aptr (cdr aptr))))))
    (modify-frame-parameters 
     frame (list (cons 'icon-type icon-form)))))

(defadvice select-window (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice other-window (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice delete-window (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice bury-buffer (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice switch-to-buffer (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice switch-to-buffer-other-window (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

(defadvice pop-to-buffer (after set-icon activate)
  "Set the icon of the frame according to filename."
  (ns-set-frame-icon))

